const renderGamepadDelay = 10;

// Internal gamepad DOM object
let gamepad = null;
// Callback timeout handle for updating the DOM
let timeout = null;

window.addEventListener("gamepadconnected", function(event) {
    if(timeout) {
        clearTimeout(timeout);
        timeout = null;
    }
    const pad = navigator.getGamepads()[event.gamepad.index];
    console.log("Gamepad connected: ", pad);
    timeout = setTimeout(renderGamepads, renderGamepadDelay);
});

window.addEventListener("gamepaddisconnected", function(event) {
    console.log("Gamepad disconnected.");
    pad = null;
    if(timeout) {
        clearTimeout(timeout);
        timeout = null;
    }
});

function renderGamepads() {
    const pads = navigator.getGamepads()
    const pad = pads[0];
    if(!pad) {
        return;
    }
    if(!gamepad) {
        gamepad = initializeGamepadXbox();
    }
    for(const key in gamepad) {
        const value = gamepad[key];
        if(typeof(value.buttonIndex) === "number") {
            if(!value.isTrigger ?
                pad.buttons[value.buttonIndex].pressed :
                pad.buttons[value.buttonIndex].value >= 1
            ) {
                value.element.classList.add("pressed");
            }
            else {
                value.element.classList.remove("pressed");
            }
            if(pad.buttons[value.buttonIndex].value >= 0.6) {
                value.element.classList.add("mostly-pressed");
            }
            else {
                value.element.classList.remove("mostly-pressed");
            }
            if(value.isTrigger && value.indicator) {
                const x = pad.buttons[value.buttonIndex].value;
                if(x) console.log(x);
                value.indicator.style.height = (x * 100) + "%";
            }
        }
        if(
            typeof(value.hAxisIndex) === "number" &&
            typeof(value.vAxisIndex) === "number"
        ) {
            let h = pad.axes[value.hAxisIndex];
            let v = pad.axes[value.vAxisIndex];
            if(h >= -0.175 && h <= 0.175) h = 0;
            if(v >= -0.175 && v <= 0.175) v = 0;
            value.indicator.style.top = (h * 50 + 50) + "%";
            value.indicator.style.left = (v * 50 + 50) + "%";
        }
    }
    timeout = setTimeout(renderGamepads, renderGamepadDelay);
}

function initializeGamepadXbox() {
    gamepad = {};
    const content = document.getElementById("content");
    const root = document.createElement("div");
    const dpad = document.createElement("div");
    const face = document.createElement("div");
    root.appendChild(dpad);
    root.appendChild(face);
    gamepad.root = root;
    gamepad.dpad = dpad;
    gamepad.face = face;
    gamepad.root.classList.add("gamepad");
    gamepad.root.classList.add("gamepad-xbox");
    gamepad.dpad.classList.add("dpad");
    gamepad.face.classList.add("face");
    document.body.appendChild(gamepad.root);
    gamepad.back = createGamepadButtonElement(root, 8, "◅", "back");
    gamepad.start = createGamepadButtonElement(root, 9, "▻", "start");
    gamepad.lshoulder = createGamepadButtonElement(root, 4, "LB", "shoulder-left");
    gamepad.rshoulder = createGamepadButtonElement(root, 5, "RB", "shoulder-right");
    gamepad.ltrigger = createGamepadTriggerElement(root, 6, "LT", "trigger-left");
    gamepad.rtrigger = createGamepadTriggerElement(root, 7, "RT", "trigger-right");
    gamepad.y = createGamepadButtonElement(face, 3, "Y", "y");
    gamepad.x = createGamepadButtonElement(face, 2, "X", "x");
    gamepad.b = createGamepadButtonElement(face, 1, "B", "b");
    gamepad.a = createGamepadButtonElement(face, 0, "A", "a");
    gamepad.up = createGamepadButtonElement(dpad, 12, "▲", "up");
    gamepad.left = createGamepadButtonElement(dpad, 14, "◄", "left");
    gamepad.right = createGamepadButtonElement(dpad, 15, "►", "right");
    gamepad.down = createGamepadButtonElement(dpad, 13, "▼", "down");
    gamepad.lstick = createGamepadStickElement(root, 10, 1, 0, "stick-left");
    gamepad.rstick = createGamepadStickElement(root, 11, 3, 2, "stick-right");
    // L stick: 10
    // R stick: 11
    return gamepad;
}

function createGamepadButtonElement(root, buttonIndex, text, className) {
    const button = document.createElement("div");
    root.appendChild(button);
    button.classList.add("gamepad-button");
    if(className) {
        button.classList.add(className);
    }
    button.innerText = text;
    return {
        element: button,
        buttonIndex: buttonIndex,
    };
}

function createGamepadStickElement(
    root, buttonIndex, hAxisIndex, vAxisIndex, className
) {
    const stick = document.createElement("div");
    root.appendChild(stick);
    stick.classList.add("gamepad-stick");
    const indicator = document.createElement("div");
    indicator.classList.add("gamepad-stick-indicator");
    stick.appendChild(indicator);
    if(className) {
        stick.classList.add(className);
    }
    return {
        element: stick,
        indicator: indicator,
        buttonIndex: buttonIndex,
        hAxisIndex: hAxisIndex,
        vAxisIndex: vAxisIndex,
    };
}

function createGamepadTriggerElement(
    root, buttonIndex, text, className
) {
    const trigger = document.createElement("div");
    root.appendChild(trigger);
    trigger.classList.add("gamepad-trigger");
    const indicator = document.createElement("div");
    indicator.classList.add("gamepad-trigger-indicator");
    const label = document.createElement("div");
    label.classList.add("gamepad-trigger-label");
    label.innerText = text;
    trigger.appendChild(indicator);
    trigger.appendChild(label);
    if(className) {
        trigger.classList.add(className);
    }
    return {
        isTrigger: true,
        element: trigger,
        indicator: indicator,
        label: label,
        buttonIndex: buttonIndex,
    };
}
